@@ -50,6 +50,7 @@ static SectionInfo section_infos[] = {
     [MMC_S_MODEM_3GPP_USSD]            = { "3GPP USSD"            },
     [MMC_S_MODEM_3GPP_PROFILE_MANAGER] = { "3GPP profile manager" },
     [MMC_S_MODEM_CDMA]                 = { "CDMA"                 },
+    [MMC_S_MODEM_CELL_BROADCAST]       = { "Cell Broadcast"       },
     [MMC_S_MODEM_SIM]                  = { "SIM"                  },
     [MMC_S_MODEM_BEARER]               = { "Bearer"               },
     [MMC_S_MODEM_TIME]                 = { "Time"                 },
@@ -87,6 +88,9 @@ static SectionInfo section_infos[] = {
     [MMC_S_SMS_PROPERTIES]             = { "Properties"           },
     [MMC_S_SIM_GENERAL]                = { "General"              },
     [MMC_S_SIM_PROPERTIES]             = { "Properties"           },
+    [MMC_S_CBM_GENERAL]                = { "General"              },
+    [MMC_S_CBM_CONTENT]                = { "Content"              },
+    [MMC_S_CBM_PROPERTIES]             = { "Properties"           },
 };
 
 /******************************************************************************/
@@ -111,6 +115,7 @@ static FieldInfo field_infos[] = {
     [MMC_F_HARDWARE_CURRENT_CAPABILITIES]            = { "modem.generic.current-capabilities",              "current",                  MMC_S_MODEM_HARDWARE,             },
     [MMC_F_HARDWARE_EQUIPMENT_ID]                    = { "modem.generic.equipment-identifier",              "equipment id",             MMC_S_MODEM_HARDWARE,             },
     [MMC_F_SYSTEM_DEVICE]                            = { "modem.generic.device",                            "device",                   MMC_S_MODEM_SYSTEM,               },
+    [MMC_F_SYSTEM_PHYSDEV]                           = { "modem.generic.physdev",                           "physdev",                  MMC_S_MODEM_SYSTEM,               },
     [MMC_F_SYSTEM_DRIVERS]                           = { "modem.generic.drivers",                           "drivers",                  MMC_S_MODEM_SYSTEM,               },
     [MMC_F_SYSTEM_PLUGIN]                            = { "modem.generic.plugin",                            "plugin",                   MMC_S_MODEM_SYSTEM,               },
     [MMC_F_SYSTEM_PRIMARY_PORT]                      = { "modem.generic.primary-port",                      "primary port",             MMC_S_MODEM_SYSTEM,               },
@@ -135,6 +140,10 @@ static FieldInfo field_infos[] = {
     [MMC_F_3GPP_OPERATOR_ID]                         = { "modem.3gpp.operator-code",                        "operator id",              MMC_S_MODEM_3GPP,                 },
     [MMC_F_3GPP_OPERATOR_NAME]                       = { "modem.3gpp.operator-name",                        "operator name",            MMC_S_MODEM_3GPP,                 },
     [MMC_F_3GPP_REGISTRATION]                        = { "modem.3gpp.registration-state",                   "registration",             MMC_S_MODEM_3GPP,                 },
+    [MMC_F_3GPP_NETWORK_REJECTION_ERROR]             = { "modem.3gpp.network-rejection-error",              "network rejection error",  MMC_S_MODEM_3GPP,                 },
+    [MMC_F_3GPP_NETWORK_REJECTION_OPERATOR_ID]       = { "modem.3gpp.network-rejection-operator-id",        "network rejection operator id",    MMC_S_MODEM_3GPP,         },
+    [MMC_F_3GPP_NETWORK_REJECTION_OPERATOR_NAME]     = { "modem.3gpp.network-rejection-operator-name",      "network rejection operator name",    MMC_S_MODEM_3GPP,       },
+    [MMC_F_3GPP_NETWORK_REJECTION_ACCESS_TECHNOLOGY] = { "modem.3gpp.network-rejection-access-technology",  "network rejection access technology",    MMC_S_MODEM_3GPP,   },
     [MMC_F_3GPP_PACKET_SERVICE_STATE]                = { "modem.3gpp.packet-service-state",                 "packet service state",     MMC_S_MODEM_3GPP,                 },
     [MMC_F_3GPP_PCO]                                 = { "modem.3gpp.pco",                                  "pco",                      MMC_S_MODEM_3GPP,                 },
     [MMC_F_3GPP_EPS_UE_MODE]                         = { "modem.3gpp.eps.ue-mode-operation",                "ue mode of operation",     MMC_S_MODEM_3GPP_EPS,             },
@@ -159,6 +168,7 @@ static FieldInfo field_infos[] = {
     [MMC_F_CDMA_REGISTRATION_CDMA1X]                 = { "modem.cdma.cdma1x-registration-state",            "registration cdma1x",      MMC_S_MODEM_CDMA,                 },
     [MMC_F_CDMA_REGISTRATION_EVDO]                   = { "modem.cdma.evdo-registration-state",              "registration evdo",        MMC_S_MODEM_CDMA,                 },
     [MMC_F_CDMA_ACTIVATION]                          = { "modem.cdma.activation-state",                     "activation",               MMC_S_MODEM_CDMA,                 },
+    [MMC_F_CELL_BROADCAST_CHANNELS]                  = { "modem.cellbroadcast.channels",                    "channels",                 MMC_S_MODEM_CELL_BROADCAST,       },
     [MMC_F_SIM_PATH]                                 = { "modem.generic.sim",                               "primary sim path",         MMC_S_MODEM_SIM,                  },
     [MMC_F_SIM_PRIMARY_SLOT]                         = { "modem.generic.primary-sim-slot",                  NULL,                       MMC_S_MODEM_SIM,                  },
     [MMC_F_SIM_SLOT_PATHS]                           = { "modem.generic.sim-slots",                         "sim slot paths",           MMC_S_MODEM_SIM,                  },
@@ -169,6 +179,7 @@ static FieldInfo field_infos[] = {
     [MMC_F_TIMEZONE_LEAP_SECONDS]                    = { "modem.time.leap-seconds",                         "leap seconds",             MMC_S_MODEM_TIMEZONE,             },
     [MMC_F_MESSAGING_SUPPORTED_STORAGES]             = { "modem.messaging.supported-storages",              "supported storages",       MMC_S_MODEM_MESSAGING,            },
     [MMC_F_MESSAGING_DEFAULT_STORAGES]               = { "modem.messaging.default-storages",                "default storages",         MMC_S_MODEM_MESSAGING,            },
+    [MMC_F_MESSAGING_CREATED_SMS]                    = { "modem.messaging.created-sms",                     "created sms",              MMC_S_MODEM_MESSAGING,            },
     [MMC_F_SIGNAL_REFRESH_RATE]                      = { "modem.signal.refresh.rate",                       "refresh rate",             MMC_S_MODEM_SIGNAL,               },
     [MMC_F_SIGNAL_RSSI_THRESHOLD]                    = { "modem.signal.threshold.rssi",                     "rssi threshold",           MMC_S_MODEM_SIGNAL,               },
     [MMC_F_SIGNAL_ERROR_RATE_THRESHOLD]              = { "modem.signal.threshold.error-rate",               "error rate threshold",     MMC_S_MODEM_SIGNAL,               },
@@ -191,6 +202,7 @@ static FieldInfo field_infos[] = {
     [MMC_F_SIGNAL_LTE_RSRP]                          = { "modem.signal.lte.rsrp",                           "rsrp",                     MMC_S_MODEM_SIGNAL_LTE,           },
     [MMC_F_SIGNAL_LTE_SNR]                           = { "modem.signal.lte.snr",                            "s/n",                      MMC_S_MODEM_SIGNAL_LTE,           },
     [MMC_F_SIGNAL_LTE_ERROR_RATE]                    = { "modem.signal.lte.error-rate",                     "error rate",               MMC_S_MODEM_SIGNAL_LTE,           },
+    [MMC_F_SIGNAL_5G_RSSI]                           = { "modem.signal.5g.rssi",                            "rssi",                     MMC_S_MODEM_SIGNAL_5G,            },
     [MMC_F_SIGNAL_5G_RSRQ]                           = { "modem.signal.5g.rsrq",                            "rsrq",                     MMC_S_MODEM_SIGNAL_5G,            },
     [MMC_F_SIGNAL_5G_RSRP]                           = { "modem.signal.5g.rsrp",                            "rsrp",                     MMC_S_MODEM_SIGNAL_5G,            },
     [MMC_F_SIGNAL_5G_SNR]                            = { "modem.signal.5g.snr",                             "s/n",                      MMC_S_MODEM_SIGNAL_5G,            },
@@ -279,6 +291,11 @@ static FieldInfo field_infos[] = {
     [MMC_F_CALL_AUDIO_FORMAT_ENCODING]               = { "call.audio-format.encoding",                      "encoding",                 MMC_S_CALL_AUDIO_FORMAT,          },
     [MMC_F_CALL_AUDIO_FORMAT_RESOLUTION]             = { "call.audio-format.resolution",                    "resolution",               MMC_S_CALL_AUDIO_FORMAT,          },
     [MMC_F_CALL_AUDIO_FORMAT_RATE]                   = { "call.audio-format.rate",                          "rate",                     MMC_S_CALL_AUDIO_FORMAT,          },
+    [MMC_F_CBM_GENERAL_DBUS_PATH]                    = { "cbm.dbus-path",                                   "path",                     MMC_S_CBM_GENERAL,                },
+    [MMC_F_CBM_CONTENT_TEXT]                         = { "cbm.content.text",                                "text",                     MMC_S_CBM_CONTENT,                },
+    [MMC_F_CBM_PROPERTIES_CHANNEL]                   = { "cbm.properties.channel",                          "channel",                  MMC_S_CBM_PROPERTIES,             },
+    [MMC_F_CBM_PROPERTIES_UPDATE]                    = { "cbm.properties.update",                           "update",                   MMC_S_CBM_PROPERTIES,             },
+    [MMC_F_CBM_PROPERTIES_MESSAGE_CODE]              = { "cbm.properties.message-code",                     "message code",             MMC_S_CBM_PROPERTIES,             },
     [MMC_F_SMS_GENERAL_DBUS_PATH]                    = { "sms.dbus-path",                                   "path",                     MMC_S_SMS_GENERAL,                },
     [MMC_F_SMS_CONTENT_NUMBER]                       = { "sms.content.number",                              "number",                   MMC_S_SMS_CONTENT,                },
     [MMC_F_SMS_CONTENT_TEXT]                         = { "sms.content.text",                                "text",                     MMC_S_SMS_CONTENT,                },
@@ -315,6 +332,7 @@ static FieldInfo field_infos[] = {
     [MMC_F_MODEM_LIST_DBUS_PATH]                     = { "modem-list",                                      "modems",                   MMC_S_UNKNOWN,                    },
     [MMC_F_SMS_LIST_DBUS_PATH]                       = { "modem.messaging.sms",                             "sms messages",             MMC_S_UNKNOWN,                    },
     [MMC_F_CALL_LIST_DBUS_PATH]                      = { "modem.voice.call",                                "calls",                    MMC_S_UNKNOWN,                    },
+    [MMC_F_CBM_LIST_DBUS_PATH]                       = { "modem.cell-broadcast.cbm",                        "cell broadcast messages",  MMC_S_UNKNOWN,                    },
 };
 
 /******************************************************************************/
@@ -357,6 +375,7 @@ typedef struct {
     OutputItem   base;
     gchar      **values;
     gboolean     multiline;
+    gboolean     is_object;
 } OutputItemMultiple;
 
 typedef struct {
@@ -418,7 +437,8 @@ output_item_new_take_single (MmcF   field,
 static void
 output_item_new_take_multiple (MmcF       field,
                                gchar    **values,
-                               gboolean   multiline)
+                               gboolean   multiline,
+                               gboolean   is_object)
 {
     OutputItemMultiple *item;
 
@@ -426,6 +446,7 @@ output_item_new_take_multiple (MmcF       field,
     item->base.field = field;
     item->base.type = VALUE_TYPE_MULTIPLE;
     item->multiline = multiline;
+    item->is_object = is_object;
 
     if (values && (g_strv_length (values) == 1) && filter_out_value (values[0]))
         g_strfreev (values);
@@ -466,7 +487,7 @@ mmcli_output_string_list (MmcF         field,
             g_strstrip (split[i]);
     }
 
-    output_item_new_take_multiple (field, split, FALSE);
+    output_item_new_take_multiple (field, split, FALSE, FALSE);
 }
 
 void
@@ -490,7 +511,7 @@ mmcli_output_string_multiline (MmcF         field,
             g_strstrip (split[i]);
     }
 
-    output_item_new_take_multiple (field, split, TRUE);
+    output_item_new_take_multiple (field, split, TRUE, FALSE);
 }
 
 void
@@ -506,7 +527,7 @@ mmcli_output_string_array (MmcF          field,
                            const gchar **strv,
                            gboolean      multiline)
 {
-    output_item_new_take_multiple (field, g_strdupv ((gchar **)strv), multiline);
+    output_item_new_take_multiple (field, g_strdupv ((gchar **)strv), multiline, FALSE);
 }
 
 void
@@ -514,7 +535,7 @@ mmcli_output_string_array_take (MmcF       field,
                                 gchar    **strv,
                                 gboolean   multiline)
 {
-    output_item_new_take_multiple (field, strv, multiline);
+    output_item_new_take_multiple (field, strv, multiline, FALSE);
 }
 
 void
@@ -550,7 +571,7 @@ mmcli_output_string_array_multiline_take (MmcF       field,
         merged = (gchar **)g_ptr_array_free (pointers, FALSE);
     }
 
-    output_item_new_take_multiple (field, merged, TRUE);
+    output_item_new_take_multiple (field, merged, TRUE, FALSE);
 }
 
 void
@@ -702,6 +723,8 @@ mmcli_output_sim_slots (gchar **sim_slot_paths,
 /******************************************************************************/
 /* (Custom) Network scan output */
 
+static gchar *json_strescape (const gchar *str);
+
 static gchar *
 build_network_info (MMModem3gppNetwork *network)
 {
@@ -718,18 +741,49 @@ build_network_info (MMModem3gppNetwork *network)
     access_technologies = mm_modem_access_technology_build_string_from_mask (mm_modem_3gpp_network_get_access_technology (network));
     availability = mm_modem_3gpp_network_availability_get_string (mm_modem_3gpp_network_get_availability (network));
 
-    if (selected_type == MMC_OUTPUT_TYPE_HUMAN)
+    if (selected_type == MMC_OUTPUT_TYPE_HUMAN) {
         out = g_strdup_printf ("%s - %s (%s, %s)",
                                operator_code ? operator_code : "code n/a",
                                operator_name ? operator_name : "name n/a",
                                access_technologies,
                                availability);
-    else
+    } else if (selected_type == MMC_OUTPUT_TYPE_JSON) {
+        GString *str;
+        gchar *escape;
+
+        str = g_string_new (NULL);
+
+        g_string_append_printf (str, "{");
+
+        escape = json_strescape (operator_code);
+        g_string_append_printf (str, "\"operator-code\":\"%s\"", escape ? escape: "--");
+        g_free (escape);
+        g_string_append_printf (str, ",");
+
+        escape = json_strescape (operator_name);
+        g_string_append_printf (str, "\"operator-name\":\"%s\"", escape ? escape: "--");
+        g_free (escape);
+        g_string_append_printf (str, ",");
+
+        escape = json_strescape (access_technologies);
+        g_string_append_printf (str, "\"access-technologies\":\"%s\"", escape);
+        g_free (escape);
+        g_string_append_printf (str, ",");
+
+        escape = json_strescape (availability);
+        g_string_append_printf (str, "\"availability\":\"%s\"", escape);
+        g_free (escape);
+
+        g_string_append_printf (str, "}");
+
+        out = g_string_free (str, FALSE);
+    } else {
         out = g_strdup_printf ("operator-code: %s, operator-name: %s, access-technologies: %s, availability: %s",
                                operator_code ? operator_code : "--",
                                operator_name ? operator_name : "--",
                                access_technologies,
                                availability);
+    }
     g_free (access_technologies);
 
     return out;
@@ -756,7 +810,7 @@ mmcli_output_scan_networks (GList *network_list)
     if (selected_type == MMC_OUTPUT_TYPE_HUMAN && !networks)
         output_item_new_take_single (MMC_F_3GPP_SCAN_NETWORKS, g_strdup ("n/a"));
     else
-        output_item_new_take_multiple (MMC_F_3GPP_SCAN_NETWORKS, networks, TRUE);
+        output_item_new_take_multiple (MMC_F_3GPP_SCAN_NETWORKS, networks, TRUE, TRUE);
 }
 
 /******************************************************************************/
@@ -848,7 +902,7 @@ mmcli_output_firmware_list (GList                *firmware_list,
     if (selected_type == MMC_OUTPUT_TYPE_HUMAN && !firmwares)
         output_item_new_take_single (MMC_F_FIRMWARE_LIST, g_strdup ("n/a"));
     else
-        output_item_new_take_multiple (MMC_F_FIRMWARE_LIST, firmwares, TRUE);
+        output_item_new_take_multiple (MMC_F_FIRMWARE_LIST, firmwares, TRUE, FALSE);
 }
 
 /******************************************************************************/
@@ -877,12 +931,12 @@ mmcli_output_pco_list (GList *pco_list)
         pco_data_hex = (pco_data ? mm_utils_bin2hexstr (pco_data, pco_data_size) : NULL);
 
         if (selected_type == MMC_OUTPUT_TYPE_HUMAN)
-            g_ptr_array_add (aux, g_strdup_printf ("%u: (%s) '%s'\n",
+            g_ptr_array_add (aux, g_strdup_printf ("%u: (%s) '%s'",
                                                    mm_pco_get_session_id (pco),
                                                    mm_pco_is_complete (pco) ? "complete" : "partial",
                                                    pco_data_hex ? pco_data_hex : ""));
         else
-            g_ptr_array_add (aux, g_strdup_printf ("session-id: %u, complete: %s, data: %s\n",
+            g_ptr_array_add (aux, g_strdup_printf ("session-id: %u, complete: %s, data: %s",
                                                    mm_pco_get_session_id (pco),
                                                    mm_pco_is_complete (pco) ? "yes" : "no",
                                                    pco_data_hex ? pco_data_hex : ""));
@@ -890,7 +944,7 @@ mmcli_output_pco_list (GList *pco_list)
     }
     g_ptr_array_add (aux, NULL);
 
-    output_item_new_take_multiple (MMC_F_3GPP_PCO, (gchar **) g_ptr_array_free (aux, FALSE), TRUE);
+    output_item_new_take_multiple (MMC_F_3GPP_PCO, (gchar **) g_ptr_array_free (aux, FALSE), TRUE, FALSE);
 }
 
 /******************************************************************************/
@@ -938,72 +992,17 @@ static void
 build_profile_human (GPtrArray     *array,
                      MM3gppProfile *profile)
 {
-    const gchar                  *aux;
-    MMBearerAllowedAuth           allowed_auth;
-    MMBearerIpFamily              ip_type;
-    MMBearerApnType               apn_type;
-    MMBearerAccessTypePreference  access_type_preference;
-    MMBearerRoamingAllowance      roaming_allowance;
-    MMBearerProfileSource         profile_source;
+    g_autoptr(GPtrArray) profile_print = NULL;
+    guint                i;
 
-    g_ptr_array_add (array, g_strdup_printf ("profile-id: %u", mm_3gpp_profile_get_profile_id (profile)));
-    g_ptr_array_add (array, g_strdup_printf ("  profile enabled: %s",
-                                             mm_3gpp_profile_get_enabled (profile) ? "yes" : "no"));
-
-    if ((aux = mm_3gpp_profile_get_profile_name (profile)) != NULL)
-        g_ptr_array_add (array, g_strdup_printf ("  profile name: %s", aux));
-
-    if ((aux = mm_3gpp_profile_get_apn (profile)) != NULL)
-        g_ptr_array_add (array, g_strdup_printf ("  apn: %s", aux));
-
-    allowed_auth = mm_3gpp_profile_get_allowed_auth (profile);
-    if (allowed_auth != MM_BEARER_ALLOWED_AUTH_NONE) {
-        g_autofree gchar *allowed_auth_str = NULL;
-
-        allowed_auth_str = mm_bearer_allowed_auth_build_string_from_mask (allowed_auth);
-        g_ptr_array_add (array, g_strdup_printf ("  allowed auth: %s", allowed_auth_str));
-    }
-
-    if ((aux = mm_3gpp_profile_get_user (profile)) != NULL)
-        g_ptr_array_add (array, g_strdup_printf ("  user: %s", aux));
-
-    if ((aux = mm_3gpp_profile_get_password (profile)) != NULL)
-        g_ptr_array_add (array, g_strdup_printf ("  password: %s", aux));
-
-    ip_type = mm_3gpp_profile_get_ip_type (profile);
-    if (ip_type != MM_BEARER_IP_FAMILY_NONE) {
-        g_autofree gchar *ip_type_str = NULL;
-
-        ip_type_str = mm_bearer_ip_family_build_string_from_mask (ip_type);
-        g_ptr_array_add (array, g_strdup_printf ("  ip type: %s", ip_type_str));
-    }
-
-    apn_type = mm_3gpp_profile_get_apn_type (profile);
-    if (apn_type != MM_BEARER_APN_TYPE_NONE) {
-        g_autofree gchar *apn_type_str = NULL;
-
-        apn_type_str = mm_bearer_apn_type_build_string_from_mask (apn_type);
-        g_ptr_array_add (array, g_strdup_printf ("  apn type: %s", apn_type_str));
-    }
-
-    access_type_preference = mm_3gpp_profile_get_access_type_preference (profile);
-    if (access_type_preference != MM_BEARER_ACCESS_TYPE_PREFERENCE_NONE) {
-        aux = mm_bearer_access_type_preference_get_string (access_type_preference);
-        g_ptr_array_add (array, g_strdup_printf ("  access type preference: %s", aux));
-    }
-
-    roaming_allowance = mm_3gpp_profile_get_roaming_allowance (profile);
-    if (roaming_allowance != MM_BEARER_ROAMING_ALLOWANCE_NONE) {
-        g_autofree gchar *roaming_allowance_str = NULL;
-
-        roaming_allowance_str = mm_bearer_roaming_allowance_build_string_from_mask (roaming_allowance);
-        g_ptr_array_add (array, g_strdup_printf ("  roaming allowance: %s", roaming_allowance_str));
-    }
-
-    profile_source = mm_3gpp_profile_get_profile_source (profile);
-    if (profile_source != MM_BEARER_PROFILE_SOURCE_UNKNOWN) {
-        aux = mm_bearer_profile_source_get_string (profile_source);
-        g_ptr_array_add (array, g_strdup_printf ("  profile source: %s", aux));
+    profile_print = mm_3gpp_profile_print (profile, TRUE);
+    mm_common_str_array_human_keys (profile_print);
+    for (i = 0; i < profile_print->len; i++) {
+        /* First string is profile id always, all the remaining ones will be
+         * indented some whitespaces right */
+        g_ptr_array_add (array, g_strdup_printf ("%s%s",
+                                                 i == 0 ? "" : "  ",
+                                                 (const gchar *)g_ptr_array_index (profile_print, i)));
     }
 }
 
@@ -1011,67 +1010,17 @@ static void
 build_profile_keyvalue (GPtrArray     *array,
                         MM3gppProfile *profile)
 {
-    GString                      *str;
-    const gchar                  *aux;
-    MMBearerAllowedAuth           allowed_auth;
-    MMBearerIpFamily              ip_type;
-    MMBearerApnType               apn_type;
-    MMBearerAccessTypePreference  access_type_preference;
-    MMBearerRoamingAllowance      roaming_allowance;
+    g_autoptr(GPtrArray)  profile_print = NULL;
+    guint                 i;
+    GString              *str;
 
     str = g_string_new ("");
-    g_string_append_printf (str, "profile-id: %u", mm_3gpp_profile_get_profile_id (profile));
-
-    if ((aux = mm_3gpp_profile_get_profile_name (profile)) != NULL)
-        g_string_append_printf (str, ", profile-name: %s", aux);
-
-    if ((aux = mm_3gpp_profile_get_apn (profile)) != NULL)
-        g_string_append_printf (str, ", apn: %s", aux);
-
-    allowed_auth = mm_3gpp_profile_get_allowed_auth (profile);
-    if (allowed_auth != MM_BEARER_ALLOWED_AUTH_NONE) {
-        g_autofree gchar *allowed_auth_str = NULL;
-
-        allowed_auth_str = mm_bearer_allowed_auth_build_string_from_mask (allowed_auth);
-        g_string_append_printf (str, ", allowed-auth: %s", allowed_auth_str);
+    profile_print = mm_3gpp_profile_print (profile, TRUE);
+    for (i = 0; i < profile_print->len; i++) {
+        g_string_append_printf (str, "%s%s",
+                                i == 0 ? "" : ", ",
+                                (const gchar *)g_ptr_array_index (profile_print, i));
     }
-
-    if ((aux = mm_3gpp_profile_get_user (profile)) != NULL)
-        g_string_append_printf (str, ", user: %s", aux);
-
-    if ((aux = mm_3gpp_profile_get_password (profile)) != NULL)
-        g_string_append_printf (str, ", password: %s", aux);
-
-    ip_type = mm_3gpp_profile_get_ip_type (profile);
-    if (ip_type != MM_BEARER_IP_FAMILY_NONE) {
-        g_autofree gchar *ip_type_str = NULL;
-
-        ip_type_str = mm_bearer_ip_family_build_string_from_mask (ip_type);
-        g_string_append_printf (str, ", ip-type: %s", ip_type_str);
-    }
-
-    apn_type = mm_3gpp_profile_get_apn_type (profile);
-    if (apn_type != MM_BEARER_APN_TYPE_NONE) {
-        g_autofree gchar *apn_type_str = NULL;
-
-        apn_type_str = mm_bearer_apn_type_build_string_from_mask (apn_type);
-        g_string_append_printf (str, ", apn-type: %s", apn_type_str);
-    }
-
-    access_type_preference = mm_3gpp_profile_get_access_type_preference (profile);
-    if (access_type_preference != MM_BEARER_ACCESS_TYPE_PREFERENCE_NONE) {
-        aux = mm_bearer_access_type_preference_get_string (access_type_preference);
-        g_string_append_printf (str, ", access-type-preference: %s", aux);
-    }
-
-    roaming_allowance = mm_3gpp_profile_get_roaming_allowance (profile);
-    if (roaming_allowance != MM_BEARER_ROAMING_ALLOWANCE_NONE) {
-        g_autofree gchar *roaming_allowance_str = NULL;
-
-        roaming_allowance_str = mm_bearer_roaming_allowance_build_string_from_mask (roaming_allowance);
-        g_string_append_printf (str, ", roaming-allowance: %s", roaming_allowance_str);
-    }
-
     g_ptr_array_add (array, g_string_free (str, FALSE));
 }
 
@@ -1103,7 +1052,7 @@ output_profile_list (MmcF   field,
     if (selected_type == MMC_OUTPUT_TYPE_HUMAN && !profiles)
         output_item_new_take_single (field, g_strdup ("n/a"));
     else
-        output_item_new_take_multiple (field, profiles, TRUE);
+        output_item_new_take_multiple (field, profiles, TRUE, FALSE);
 }
 
 void
@@ -1146,7 +1095,7 @@ mmcli_output_cell_info (GList *cell_info_list)
     if (selected_type == MMC_OUTPUT_TYPE_HUMAN && !cell_infos)
         output_item_new_take_single (MMC_F_CELL_INFO, g_strdup ("n/a"));
     else
-        output_item_new_take_multiple (MMC_F_CELL_INFO, cell_infos, TRUE);
+        output_item_new_take_multiple (MMC_F_CELL_INFO, cell_infos, TRUE, FALSE);
 }
 
 /******************************************************************************/
@@ -1540,7 +1489,7 @@ dump_output_json (void)
 
     output_items = g_list_sort (output_items, (GCompareFunc) list_sort_by_keys);
 
-    g_print("{");
+    g_print ("{");
     for (l = output_items; l; l = g_list_next (l)) {
         OutputItem *item_l = (OutputItem *)(l->data);
 
@@ -1594,22 +1543,26 @@ dump_output_json (void)
 
             g_print ("\"%s\":[", current_path[cur_dlen]);
             for (i = 0; i < n; i++) {
-                gchar *escaped;
+                if (multiple->is_object) {
+                    g_print ("%s", multiple->values[i]);
+                } else {
+                    gchar *escaped;
 
-                escaped = json_strescape (multiple->values[i]);
-                g_print("\"%s\"", escaped);
+                    escaped = json_strescape (multiple->values[i]);
+                    g_print ("\"%s\"", escaped);
+                    g_free (escaped);
+                }
                 if (i < n - 1)
-                    g_print(",");
-                g_free (escaped);
+                    g_print (",");
             }
-            g_print("]");
+            g_print ("]");
         } else
             g_assert_not_reached ();
     }
 
     while (cur_dlen--)
         g_print ("}");
-    g_print("}\n");
+    g_print ("}\n");
 
     g_strfreev (current_path);
 }
@@ -1621,7 +1574,7 @@ dump_output_list_json (MmcF field)
 
     g_assert (field != MMC_F_UNKNOWN);
 
-    g_print("{\"%s\":[", field_infos[field].key);
+    g_print ("{\"%s\":[", field_infos[field].key);
 
     for (l = output_items; l; l = g_list_next (l)) {
         OutputItem         *item_l;
@@ -1634,13 +1587,13 @@ dump_output_list_json (MmcF field)
 
         /* All items must be of same type */
         g_assert_cmpint (item_l->field, ==, field);
-        g_print("\"%s\"", listitem->value);
+        g_print ("\"%s\"", listitem->value);
 
         if (g_list_next (l))
-            g_print(",");
+            g_print (",");
     }
 
-    g_print("]}\n");
+    g_print ("]}\n");
 }
 
 /******************************************************************************/
