@@ -23,16 +23,14 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
-#define MM_TYPE_IFACE_MODEM_3GPP_PROFILE_MANAGER               (mm_iface_modem_3gpp_profile_manager_get_type ())
-#define MM_IFACE_MODEM_3GPP_PROFILE_MANAGER(obj)               (G_TYPE_CHECK_INSTANCE_CAST ((obj), MM_TYPE_IFACE_MODEM_3GPP_PROFILE_MANAGER, MMIfaceModem3gppProfileManager))
-#define MM_IS_IFACE_MODEM_3GPP_PROFILE_MANAGER(obj)            (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MM_TYPE_IFACE_MODEM_3GPP_PROFILE_MANAGER))
-#define MM_IFACE_MODEM_3GPP_PROFILE_MANAGER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MM_TYPE_IFACE_MODEM_3GPP_PROFILE_MANAGER, MMIfaceModem3gppProfileManager))
+#include "mm-iface-modem-3gpp.h"
+
+#define MM_TYPE_IFACE_MODEM_3GPP_PROFILE_MANAGER mm_iface_modem_3gpp_profile_manager_get_type ()
+G_DECLARE_INTERFACE (MMIfaceModem3gppProfileManager, mm_iface_modem_3gpp_profile_manager, MM, IFACE_MODEM_3GPP_PROFILE_MANAGER, MMIfaceModem3gpp)
 
 #define MM_IFACE_MODEM_3GPP_PROFILE_MANAGER_DBUS_SKELETON "iface-modem-3gpp-profile-manager-dbus-skeleton"
 
-typedef struct _MMIfaceModem3gppProfileManager MMIfaceModem3gppProfileManager;
-
-struct _MMIfaceModem3gppProfileManager {
+struct _MMIfaceModem3gppProfileManagerInterface {
     GTypeInterface g_iface;
 
     /* Check for profile management support (async) */
@@ -188,9 +186,6 @@ struct _MMIfaceModem3gppProfileManager {
                                        GError                         **error);
 };
 
-GType mm_iface_modem_3gpp_profile_manager_get_type (void);
-G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMIfaceModem3gppProfileManager, g_object_unref)
-
 /* Initialize profile manager interface (async) */
 void     mm_iface_modem_3gpp_profile_manager_initialize        (MMIfaceModem3gppProfileManager  *self,
                                                                 GAsyncReadyCallback              callback,
@@ -222,6 +217,12 @@ void mm_iface_modem_3gpp_profile_manager_shutdown (MMIfaceModem3gppProfileManage
 void mm_iface_modem_3gpp_profile_manager_bind_simple_status (MMIfaceModem3gppProfileManager *self,
                                                              MMSimpleStatus                 *status);
 
+/* Helpers to request ignorig Updated signal emission requests sent by implementations during
+ * our own operations */
+void mm_iface_modem_3gpp_profile_manager_update_ignore_start        (MMIfaceModem3gppProfileManager *self);
+void mm_iface_modem_3gpp_profile_manager_update_ignore_stop         (MMIfaceModem3gppProfileManager *self);
+void mm_iface_modem_3gpp_profile_manager_update_ignore_stop_delayed (MMIfaceModem3gppProfileManager *self);
+
 /* Helper to emit the Updated signal by implementations */
 void mm_iface_modem_3gpp_profile_manager_updated (MMIfaceModem3gppProfileManager *self);
 
@@ -250,5 +251,4 @@ MM3gppProfile *mm_iface_modem_3gpp_profile_manager_set_profile_finish   (MMIface
                                                                          GAsyncResult                    *res,
                                                                          GError                         **error);
 
-
 #endif /* MM_IFACE_MODEM_3GPP_PROFILE_MANAGER_H */
