@@ -20,6 +20,7 @@
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-messaging.h"
 #include "mm-sms-list.h"
+#include "mm-error-helpers.h"
 #include "mm-log-object.h"
 
 #define SUPPORT_CHECKED_TAG "messaging-support-checked-tag"
@@ -30,6 +31,8 @@ static GQuark support_checked_quark;
 static GQuark supported_quark;
 static GQuark storage_context_quark;
 
+G_DEFINE_INTERFACE (MMIfaceModemMessaging, mm_iface_modem_messaging, MM_TYPE_IFACE_MODEM)
+
 /*****************************************************************************/
 
 guint8
@@ -91,9 +94,9 @@ mm_iface_modem_messaging_bind_simple_status (MMIfaceModemMessaging *self,
 MMBaseSms *
 mm_iface_modem_messaging_create_sms (MMIfaceModemMessaging *self)
 {
-    g_assert (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->create_sms != NULL);
+    g_assert (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->create_sms != NULL);
 
-    return MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->create_sms (self);
+    return MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->create_sms (self);
 }
 
 /*****************************************************************************/
@@ -146,7 +149,7 @@ typedef struct {
     MmGdbusModemMessaging *skeleton;
     GDBusMethodInvocation *invocation;
     MMIfaceModemMessaging *self;
-    gchar *path;
+    gchar                 *path;
 } HandleDeleteContext;
 
 static void
@@ -156,53 +159,48 @@ handle_delete_context_free (HandleDeleteContext *ctx)
     g_object_unref (ctx->invocation);
     g_object_unref (ctx->self);
     g_free (ctx->path);
-    g_free (ctx);
+    g_slice_free (HandleDeleteContext, ctx);
 }
 
 static void
-handle_delete_ready (MMSmsList *list,
-                     GAsyncResult *res,
+handle_delete_ready (MMSmsList           *list,
+                     GAsyncResult        *res,
                      HandleDeleteContext *ctx)
 {
     GError *error = NULL;
 
-    if (!mm_sms_list_delete_sms_finish (list, res, &error))
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
-    else
+    if (!mm_sms_list_delete_sms_finish (list, res, &error)) {
+        mm_obj_warn (ctx->self, "failed deleting SMS message '%s': %s", ctx->path, error->message);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
+    } else {
+        mm_obj_info (ctx->self, "deleted SMS message '%s'", ctx->path);
         mm_gdbus_modem_messaging_complete_delete (ctx->skeleton, ctx->invocation);
-
+    }
     handle_delete_context_free (ctx);
 }
 
 static void
-handle_delete_auth_ready (MMBaseModem *self,
-                          GAsyncResult *res,
+handle_delete_auth_ready (MMBaseModem         *self,
+                          GAsyncResult        *res,
                           HandleDeleteContext *ctx)
 {
-    MMModemState modem_state = MM_MODEM_STATE_UNKNOWN;
-    MMSmsList *list = NULL;
-    GError *error = NULL;
+    g_autoptr(MMSmsList)  list = NULL;
+    GError               *error = NULL;
 
     if (!mm_base_modem_authorize_finish (self, res, &error)) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         handle_delete_context_free (ctx);
         return;
     }
 
-    g_object_get (self,
-                  MM_IFACE_MODEM_STATE, &modem_state,
-                  NULL);
-
     /* We do allow deleting SMS messages while enabling or disabling, it doesn't
      * interfere with the state transition logic to do so. The main reason to allow
      * this is that during modem enabling we're emitting "Added" signals before we
      * reach the enabled state, and so users listening to the signal may want to
      * delete the SMS message as soon as it's read. */
-    if (modem_state <= MM_MODEM_STATE_DISABLED) {
-        g_dbus_method_invocation_return_error (ctx->invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_WRONG_STATE,
-                                               "Cannot delete SMS: device not yet enabled");
+    if (mm_iface_modem_abort_invocation_if_state_not_reached (MM_IFACE_MODEM (self),
+                                                              ctx->invocation,
+                                                              MM_MODEM_STATE_DISABLING)) {
         handle_delete_context_free (ctx);
         return;
     }
@@ -211,30 +209,28 @@ handle_delete_auth_ready (MMBaseModem *self,
                   MM_IFACE_MODEM_MESSAGING_SMS_LIST, &list,
                   NULL);
     if (!list) {
-        g_dbus_method_invocation_return_error (ctx->invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_WRONG_STATE,
-                                               "Cannot delete SMS: missing SMS list");
+        mm_dbus_method_invocation_return_error_literal (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_WRONG_STATE,
+                                                        "Cannot delete SMS: missing SMS list");
         handle_delete_context_free (ctx);
         return;
     }
 
+    mm_obj_info (self, "processing user request to delete SMS message '%s'...", ctx->path);
     mm_sms_list_delete_sms (list,
                             ctx->path,
                             (GAsyncReadyCallback)handle_delete_ready,
                             ctx);
-    g_object_unref (list);
 }
 
 static gboolean
 handle_delete (MmGdbusModemMessaging *skeleton,
                GDBusMethodInvocation *invocation,
-               const gchar *path,
+               const gchar           *path,
                MMIfaceModemMessaging *self)
 {
     HandleDeleteContext *ctx;
 
-    ctx = g_new (HandleDeleteContext, 1);
+    ctx = g_slice_new0 (HandleDeleteContext);
     ctx->skeleton = g_object_ref (skeleton);
     ctx->invocation = g_object_ref (invocation);
     ctx->self = g_object_ref (self);
@@ -254,7 +250,7 @@ typedef struct {
     MmGdbusModemMessaging *skeleton;
     GDBusMethodInvocation *invocation;
     MMIfaceModemMessaging *self;
-    GVariant *dictionary;
+    GVariant              *dictionary;
 } HandleCreateContext;
 
 static void
@@ -264,35 +260,28 @@ handle_create_context_free (HandleCreateContext *ctx)
     g_object_unref (ctx->invocation);
     g_object_unref (ctx->self);
     g_variant_unref (ctx->dictionary);
-    g_free (ctx);
+    g_slice_free (HandleCreateContext, ctx);
 }
 
 static void
-handle_create_auth_ready (MMBaseModem *self,
-                          GAsyncResult *res,
+handle_create_auth_ready (MMBaseModem         *self,
+                          GAsyncResult        *res,
                           HandleCreateContext *ctx)
 {
-    MMModemState modem_state = MM_MODEM_STATE_UNKNOWN;
-    MMSmsList *list = NULL;
-    GError *error = NULL;
-    MMSmsProperties *properties;
-    MMBaseSms *sms;
+    GError                     *error = NULL;
+    g_autoptr(MMSmsList)        list = NULL;
+    g_autoptr(MMSmsProperties)  properties = NULL;
+    g_autoptr(MMBaseSms)        sms = NULL;
 
     if (!mm_base_modem_authorize_finish (self, res, &error)) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         handle_create_context_free (ctx);
         return;
     }
 
-    g_object_get (self,
-                  MM_IFACE_MODEM_STATE, &modem_state,
-                  NULL);
-
-    if (modem_state < MM_MODEM_STATE_ENABLED) {
-        g_dbus_method_invocation_return_error (ctx->invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_WRONG_STATE,
-                                               "Cannot create SMS: device not yet enabled");
+    if (mm_iface_modem_abort_invocation_if_state_not_reached (MM_IFACE_MODEM (self),
+                                                              ctx->invocation,
+                                                              MM_MODEM_STATE_ENABLED)) {
         handle_create_context_free (ctx);
         return;
     }
@@ -300,17 +289,14 @@ handle_create_auth_ready (MMBaseModem *self,
     /* Parse input properties */
     properties = mm_sms_properties_new_from_dictionary (ctx->dictionary, &error);
     if (!properties) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         handle_create_context_free (ctx);
         return;
     }
 
-    sms = mm_base_sms_new_from_properties (MM_BASE_MODEM (self),
-                                           properties,
-                                           &error);
+    sms = mm_base_sms_new_from_properties (MM_BASE_MODEM (self), properties, &error);
     if (!sms) {
-        g_object_unref (properties);
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         handle_create_context_free (ctx);
         return;
     }
@@ -319,40 +305,30 @@ handle_create_auth_ready (MMBaseModem *self,
                   MM_IFACE_MODEM_MESSAGING_SMS_LIST, &list,
                   NULL);
     if (!list) {
-        g_object_unref (properties);
-        g_object_unref (sms);
-        g_dbus_method_invocation_return_error (ctx->invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_WRONG_STATE,
-                                               "Cannot create SMS: missing SMS list");
+        mm_dbus_method_invocation_return_error_literal (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_WRONG_STATE,
+                                                        "Cannot create SMS: missing SMS list");
         handle_create_context_free (ctx);
         return;
     }
 
-    /* Add it to the list */
+    mm_obj_info (self, "processing user request to create SMS message...");
     mm_sms_list_add_sms (list, sms);
-
-    /* Complete the DBus call */
     mm_gdbus_modem_messaging_complete_create (ctx->skeleton,
                                               ctx->invocation,
                                               mm_base_sms_get_path (sms));
-    g_object_unref (sms);
-
-    g_object_unref (properties);
-    g_object_unref (list);
-
+    mm_obj_info (self, "created SMS message: %s", mm_base_sms_get_path (sms));
     handle_create_context_free (ctx);
 }
 
 static gboolean
 handle_create (MmGdbusModemMessaging *skeleton,
                GDBusMethodInvocation *invocation,
-               GVariant *dictionary,
+               GVariant              *dictionary,
                MMIfaceModemMessaging *self)
 {
     HandleCreateContext *ctx;
 
-    ctx = g_new (HandleCreateContext, 1);
+    ctx = g_slice_new0 (HandleCreateContext);
     ctx->skeleton = g_object_ref (skeleton);
     ctx->invocation = g_object_ref (invocation);
     ctx->self = g_object_ref (self);
@@ -373,46 +349,135 @@ handle_list (MmGdbusModemMessaging *skeleton,
              GDBusMethodInvocation *invocation,
              MMIfaceModemMessaging *self)
 {
-    GStrv paths;
-    MMSmsList *list = NULL;
-    MMModemState modem_state;
+    g_auto(GStrv)        paths = NULL;
+    g_autoptr(MMSmsList) list = NULL;
 
-    modem_state = MM_MODEM_STATE_UNKNOWN;
-    g_object_get (self,
-                  MM_IFACE_MODEM_STATE, &modem_state,
-                  NULL);
-
-    if (modem_state < MM_MODEM_STATE_ENABLED) {
-        g_dbus_method_invocation_return_error (invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_WRONG_STATE,
-                                               "Cannot list SMS messages: "
-                                               "device not yet enabled");
+    if (mm_iface_modem_abort_invocation_if_state_not_reached (MM_IFACE_MODEM (self),
+                                                              invocation,
+                                                              MM_MODEM_STATE_ENABLED))
         return TRUE;
-    }
 
     g_object_get (self,
                   MM_IFACE_MODEM_MESSAGING_SMS_LIST, &list,
                   NULL);
     if (!list) {
-        g_dbus_method_invocation_return_error (invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_WRONG_STATE,
-                                               "Cannot list SMS: missing SMS list");
+        mm_dbus_method_invocation_return_error_literal (invocation, MM_CORE_ERROR, MM_CORE_ERROR_WRONG_STATE,
+                                                        "Cannot list SMS: missing SMS list");
         return TRUE;
     }
 
+    mm_obj_info (self, "processing user request to list SMS messages...");
     paths = mm_sms_list_get_paths (list);
     mm_gdbus_modem_messaging_complete_list (skeleton,
                                             invocation,
                                             (const gchar *const *)paths);
-    g_strfreev (paths);
-    g_object_unref (list);
+    mm_obj_info (self, "reported %u SMS messages available", paths ? g_strv_length (paths) : 0);
     return TRUE;
 }
 
 /*****************************************************************************/
 
+typedef struct {
+    MmGdbusModemMessaging *skeleton;
+    GDBusMethodInvocation *invocation;
+    MMIfaceModemMessaging *self;
+    MMSmsStorage           storage;
+} HandleSetDefaultStorageContext;
+
+static void
+handle_set_default_storage_context_free (HandleSetDefaultStorageContext *ctx)
+{
+    g_object_unref (ctx->skeleton);
+    g_object_unref (ctx->invocation);
+    g_object_unref (ctx->self);
+    g_slice_free (HandleSetDefaultStorageContext, ctx);
+}
+static void
+handle_set_default_storage_ready (MMIfaceModemMessaging          *self,
+                                  GAsyncResult                   *res,
+                                  HandleSetDefaultStorageContext *ctx)
+{
+    GError *error = NULL;
+
+    if (!MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->set_default_storage_finish (self, res, &error)) {
+        mm_obj_warn (self, "could not set default storage: %s", error->message);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
+        handle_set_default_storage_context_free (ctx);
+        return;
+    }
+
+    g_object_set (self,
+                  MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE, ctx->storage,
+                  NULL);
+
+    mm_obj_info (self, "set the default storage successfully");
+    mm_gdbus_modem_messaging_complete_set_default_storage (ctx->skeleton, ctx->invocation);
+    handle_set_default_storage_context_free (ctx);
+}
+
+static void
+handle_set_default_storage_auth_ready (MMBaseModem                    *self,
+                                       GAsyncResult                   *res,
+                                       HandleSetDefaultStorageContext *ctx)
+{
+    GError *error = NULL;
+
+    if (!mm_base_modem_authorize_finish (self, res, &error)) {
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
+        handle_set_default_storage_context_free (ctx);
+        return;
+    }
+
+    /* Check if the requested storage is supported or not */
+    if (!mm_iface_modem_messaging_is_storage_supported_for_storing (ctx->self, ctx->storage, &error)) {
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
+        handle_set_default_storage_context_free (ctx);
+        return;
+    }
+
+    mm_obj_info (self, "processing user request to set default storage '%s'...",
+                 mm_sms_storage_get_string (ctx->storage));
+
+    if (!(MM_IFACE_MODEM_MESSAGING_GET_IFACE (ctx->self)->set_default_storage) ||
+        !(MM_IFACE_MODEM_MESSAGING_GET_IFACE (ctx->self)->set_default_storage_finish)) {
+        mm_dbus_method_invocation_return_error (ctx->invocation,
+                                               MM_CORE_ERROR,
+                                               MM_CORE_ERROR_UNSUPPORTED,
+                                               "Cannot set default storage: not supported");
+        handle_set_default_storage_context_free (ctx);
+        return;
+    }
+
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (ctx->self)->set_default_storage (
+        ctx->self,
+        ctx->storage,
+        (GAsyncReadyCallback)handle_set_default_storage_ready,
+        ctx);
+}
+
+static gboolean
+handle_set_default_storage (MmGdbusModemMessaging *skeleton,
+                            GDBusMethodInvocation *invocation,
+                            guint32                storage,
+                            MMIfaceModemMessaging *self)
+{
+    HandleSetDefaultStorageContext *ctx;
+
+    ctx = g_slice_new0 (HandleSetDefaultStorageContext);
+    ctx->skeleton   = g_object_ref (skeleton);
+    ctx->invocation = g_object_ref (invocation);
+    ctx->self       = g_object_ref (self);
+    ctx->storage    = (MMSmsStorage)storage;
+
+    mm_base_modem_authorize (MM_BASE_MODEM (self),
+                             invocation,
+                             MM_AUTHORIZATION_MESSAGING,
+                             (GAsyncReadyCallback)handle_set_default_storage_auth_ready,
+                             ctx);
+    return TRUE;
+}
+/*****************************************************************************/
+
 gboolean
 mm_iface_modem_messaging_take_part (MMIfaceModemMessaging *self,
                                     MMSmsPart             *sms_part,
@@ -565,7 +630,7 @@ disable_unsolicited_events_ready (MMIfaceModemMessaging *self,
     DisablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events_finish (self, res, &error);
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->disable_unsolicited_events_finish (self, res, &error);
     if (error) {
         g_task_return_error (task, error);
         g_object_unref (task);
@@ -586,7 +651,7 @@ cleanup_unsolicited_events_ready (MMIfaceModemMessaging *self,
     DisablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events_finish (self, res, &error);
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->cleanup_unsolicited_events_finish (self, res, &error);
     if (error) {
         g_task_return_error (task, error);
         g_object_unref (task);
@@ -615,9 +680,9 @@ interface_disabling_step (GTask *task)
 
     case DISABLING_STEP_DISABLE_UNSOLICITED_EVENTS:
         /* Allow cleaning up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->disable_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->disable_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->disable_unsolicited_events (
                 self,
                 (GAsyncReadyCallback)disable_unsolicited_events_ready,
                 task);
@@ -628,9 +693,9 @@ interface_disabling_step (GTask *task)
 
     case DISABLING_STEP_CLEANUP_UNSOLICITED_EVENTS:
         /* Allow cleaning up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->cleanup_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->cleanup_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->cleanup_unsolicited_events (
                 self,
                 (GAsyncReadyCallback)cleanup_unsolicited_events_ready,
                 task);
@@ -731,7 +796,7 @@ setup_sms_format_ready (MMIfaceModemMessaging *self,
     EnablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format_finish (self, res, &error);
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_sms_format_finish (self, res, &error);
     if (error) {
         g_task_return_error (task, error);
         g_object_unref (task);
@@ -756,7 +821,7 @@ load_initial_sms_parts_ready (MMIfaceModemMessaging *self,
 
     ctx = g_task_get_task_data (task);
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts_finish (self, res, &error);
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_initial_sms_parts_finish (self, res, &error);
     if (error) {
         StorageContext *storage_ctx;
 
@@ -782,7 +847,7 @@ set_default_storage_ready (MMIfaceModemMessaging *self,
     EnablingContext *ctx;
     GError *error = NULL;
 
-    if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage_finish (self, res, &error)) {
+    if (!MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->set_default_storage_finish (self, res, &error)) {
         mm_obj_warn (self, "could not set default storage: %s", error->message);
         g_error_free (error);
     }
@@ -825,7 +890,7 @@ load_initial_sms_parts_from_storages (GTask *task)
         return;
     }
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts (
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_initial_sms_parts (
         self,
         g_array_index (storage_ctx->supported_mem1,
                        MMSmsStorage,
@@ -842,7 +907,7 @@ setup_unsolicited_events_ready (MMIfaceModemMessaging *self,
     EnablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events_finish (self, res, &error);
+    MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_unsolicited_events_finish (self, res, &error);
     if (error) {
         g_task_return_error (task, error);
         g_object_unref (task);
@@ -864,7 +929,7 @@ enable_unsolicited_events_ready (MMIfaceModemMessaging *self,
     GError *error = NULL;
 
     /* Not critical! */
-    if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events_finish (self, res, &error)) {
+    if (!MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->enable_unsolicited_events_finish (self, res, &error)) {
         mm_obj_dbg (self, "couldn't enable unsolicited events: %s", error->message);
         g_error_free (error);
     }
@@ -962,9 +1027,9 @@ interface_enabling_step (GTask *task)
 
     case ENABLING_STEP_SETUP_SMS_FORMAT:
         /* Allow setting SMS format to use */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_sms_format &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_sms_format_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_sms_format (
                 self,
                 (GAsyncReadyCallback)setup_sms_format_ready,
                 task);
@@ -976,22 +1041,31 @@ interface_enabling_step (GTask *task)
     case ENABLING_STEP_STORAGE_DEFAULTS: {
         MMSmsStorage default_storage;
 
-        /* Is there only one single storage supported? if so, we don't care if
-         * setting default storage is implemented or not. */
-        default_storage = get_single_default_sms_storage (self);
-        if (default_storage == MM_SMS_STORAGE_UNKNOWN)
-            default_storage = get_best_initial_default_sms_storage (self);
-
-        /* Already bound to the 'default-storage' property in the skeleton */
-        g_object_set (self,
-                      MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE, default_storage,
+        /* Get default storage that is set by init_current_storage */
+        g_object_get (self,
+                      MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE, &default_storage,
                       NULL);
 
+        /* if current_storage is set to unknown, get the best initial
+         * default_storage */
+        if (default_storage == MM_SMS_STORAGE_UNKNOWN) {
+            /* Is there only one single storage supported? if so, we don't care if
+             * setting default storage is implemented or not. */
+            default_storage = get_single_default_sms_storage (self);
+            if (default_storage == MM_SMS_STORAGE_UNKNOWN)
+                default_storage = get_best_initial_default_sms_storage (self);
+
+            /* Already bound to the 'default-storage' property in the skeleton */
+            g_object_set (self,
+                          MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE, default_storage,
+                          NULL);
+        }
+
         if (default_storage == MM_SMS_STORAGE_UNKNOWN)
             mm_obj_warn (self, "cannot set default storage, none of the suggested ones supported");
-        else if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage &&
-                 MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage (
+        else if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->set_default_storage &&
+                 MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->set_default_storage_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->set_default_storage (
                 self,
                 default_storage,
                 (GAsyncReadyCallback)set_default_storage_ready,
@@ -1004,8 +1078,8 @@ interface_enabling_step (GTask *task)
 
     case ENABLING_STEP_LOAD_INITIAL_SMS_PARTS:
         /* Allow loading the initial list of SMS parts */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts_finish) {
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_initial_sms_parts &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_initial_sms_parts_finish) {
             load_initial_sms_parts_from_storages (task);
             return;
         }
@@ -1014,9 +1088,9 @@ interface_enabling_step (GTask *task)
 
     case ENABLING_STEP_SETUP_UNSOLICITED_EVENTS:
         /* Allow setting up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->setup_unsolicited_events (
                 self,
                 (GAsyncReadyCallback)setup_unsolicited_events_ready,
                 task);
@@ -1027,9 +1101,9 @@ interface_enabling_step (GTask *task)
 
     case ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS:
         /* Allow setting up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->enable_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->enable_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->enable_unsolicited_events (
                 self,
                 (GAsyncReadyCallback)enable_unsolicited_events_ready,
                 task);
@@ -1133,7 +1207,7 @@ load_supported_storages_ready (MMIfaceModemMessaging *self,
 
     ctx = g_task_get_task_data (task);
     storage_ctx = get_storage_context (self);
-    if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages_finish (
+    if (!MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_supported_storages_finish (
             self,
             res,
             &storage_ctx->supported_mem1,
@@ -1209,9 +1283,7 @@ check_support_ready (MMIfaceModemMessaging *self,
     InitializationContext *ctx;
     GError *error = NULL;
 
-    if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support_finish (self,
-                                                                              res,
-                                                                              &error)) {
+    if (!MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->check_support_finish (self, res, &error)) {
         if (error) {
             /* This error shouldn't be treated as critical */
             mm_obj_dbg (self, "messaging support check failed: %s", error->message);
@@ -1237,15 +1309,21 @@ init_current_storages_ready (MMIfaceModemMessaging *self,
 {
     InitializationContext *ctx;
     GError *error = NULL;
+    MMSmsStorage current_storage = MM_SMS_STORAGE_UNKNOWN;
 
-    if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages_finish (
+    if (!MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->init_current_storages_finish (
             self,
             res,
+            &current_storage,
             &error)) {
         mm_obj_dbg (self, "couldn't initialize current storages: %s", error->message);
         g_error_free (error);
-    } else
+    } else {
+        g_object_set (self,
+                      MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE, current_storage,
+                      NULL);
         mm_obj_dbg (self, "current storages initialized");
+    }
 
     /* Go on to next step */
     ctx = g_task_get_task_data (task);
@@ -1293,9 +1371,9 @@ interface_initialization_step (GTask *task)
                                 supported_quark,
                                 GUINT_TO_POINTER (FALSE));
 
-            if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support &&
-                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support_finish) {
-                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support (
+            if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->check_support &&
+                MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->check_support_finish) {
+                MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->check_support (
                     self,
                     (GAsyncReadyCallback)check_support_ready,
                     task);
@@ -1322,9 +1400,9 @@ interface_initialization_step (GTask *task)
         /* fall through */
 
     case INITIALIZATION_STEP_LOAD_SUPPORTED_STORAGES:
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_supported_storages &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_supported_storages_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->load_supported_storages (
                 self,
                 (GAsyncReadyCallback)load_supported_storages_ready,
                 task);
@@ -1334,9 +1412,9 @@ interface_initialization_step (GTask *task)
         /* fall through */
 
     case INITIALIZATION_STEP_INIT_CURRENT_STORAGES:
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages (
+        if (MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->init_current_storages &&
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->init_current_storages_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_IFACE (self)->init_current_storages (
                 self,
                 (GAsyncReadyCallback)init_current_storages_ready,
                 task);
@@ -1361,6 +1439,10 @@ interface_initialization_step (GTask *task)
                           "handle-list",
                           G_CALLBACK (handle_list),
                           self);
+        g_signal_connect (ctx->skeleton,
+                          "handle-set-default-storage",
+                          G_CALLBACK (handle_set_default_storage),
+                          self);
 
         /* Finally, export the new interface */
         mm_gdbus_object_skeleton_set_modem_messaging (MM_GDBUS_OBJECT_SKELETON (self),
@@ -1438,69 +1520,46 @@ mm_iface_modem_messaging_shutdown (MMIfaceModemMessaging *self)
 /*****************************************************************************/
 
 static void
-iface_modem_messaging_init (gpointer g_iface)
+mm_iface_modem_messaging_default_init (MMIfaceModemMessagingInterface *iface)
 {
-    static gboolean initialized = FALSE;
+    static gsize initialized = 0;
 
-    if (initialized)
+    if (!g_once_init_enter (&initialized))
         return;
 
     /* Properties */
-    g_object_interface_install_property
-        (g_iface,
-         g_param_spec_object (MM_IFACE_MODEM_MESSAGING_DBUS_SKELETON,
-                              "Messaging DBus skeleton",
-                              "DBus skeleton for the Messaging interface",
-                              MM_GDBUS_TYPE_MODEM_MESSAGING_SKELETON,
+    g_object_interface_install_property (
+        iface,
+        g_param_spec_object (MM_IFACE_MODEM_MESSAGING_DBUS_SKELETON,
+                             "Messaging DBus skeleton",
+                             "DBus skeleton for the Messaging interface",
+                             MM_GDBUS_TYPE_MODEM_MESSAGING_SKELETON,
+                             G_PARAM_READWRITE));
+
+    g_object_interface_install_property (
+        iface,
+        g_param_spec_object (MM_IFACE_MODEM_MESSAGING_SMS_LIST,
+                             "SMS list",
+                             "List of SMS objects managed in the interface",
+                             MM_TYPE_SMS_LIST,
+                             G_PARAM_READWRITE));
+
+    g_object_interface_install_property (
+        iface,
+        g_param_spec_boolean (MM_IFACE_MODEM_MESSAGING_SMS_PDU_MODE,
+                              "PDU mode",
+                              "Whether PDU mode should be used",
+                              FALSE,
                               G_PARAM_READWRITE));
 
-    g_object_interface_install_property
-        (g_iface,
-         g_param_spec_object (MM_IFACE_MODEM_MESSAGING_SMS_LIST,
-                              "SMS list",
-                              "List of SMS objects managed in the interface",
-                              MM_TYPE_SMS_LIST,
-                              G_PARAM_READWRITE));
+    g_object_interface_install_property (
+        iface,
+        g_param_spec_enum (MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE,
+                           "SMS default storage",
+                           "Default storage to be used when storing/receiving SMS messages",
+                           MM_TYPE_SMS_STORAGE,
+                           MM_SMS_STORAGE_ME,
+                           G_PARAM_READWRITE));
 
-    g_object_interface_install_property
-        (g_iface,
-         g_param_spec_boolean (MM_IFACE_MODEM_MESSAGING_SMS_PDU_MODE,
-                               "PDU mode",
-                               "Whether PDU mode should be used",
-                               FALSE,
-                               G_PARAM_READWRITE));
-
-    g_object_interface_install_property
-        (g_iface,
-         g_param_spec_enum (MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE,
-                            "SMS default storage",
-                            "Default storage to be used when storing/receiving SMS messages",
-                            MM_TYPE_SMS_STORAGE,
-                            MM_SMS_STORAGE_ME,
-                            G_PARAM_READWRITE));
-
-    initialized = TRUE;
-}
-
-GType
-mm_iface_modem_messaging_get_type (void)
-{
-    static GType iface_modem_messaging_type = 0;
-
-    if (!G_UNLIKELY (iface_modem_messaging_type)) {
-        static const GTypeInfo info = {
-            sizeof (MMIfaceModemMessaging), /* class_size */
-            iface_modem_messaging_init,     /* base_init */
-            NULL,                           /* base_finalize */
-        };
-
-        iface_modem_messaging_type = g_type_register_static (G_TYPE_INTERFACE,
-                                                             "MMIfaceModemMessaging",
-                                                             &info,
-                                                             0);
-
-        g_type_interface_add_prerequisite (iface_modem_messaging_type, MM_TYPE_IFACE_MODEM);
-    }
-
-    return iface_modem_messaging_type;
+    g_once_init_leave (&initialized, 1);
 }
